
******************************************************************;
* Module: XLSelect.sas                                           *;
* Type: SAS macro                                                *;
* Function: Select cells in an Excel worksheet                   *;
*           Supports selection of only one row                   *;
*                              or a sequence of rows             *;
*                              or only one column                *;
*                              or a sequence of columns          *;
*                              or a group of cells               *;
*                              or only one cell                  *;
* NOTE: For this macro and macros that invoke it,                *;
*       it is NOT necessary to specify WorkSheet=                *;
*       If you do not,                                           *;
*       the selection will affect the currently                  *;
*       selected or active worksheet.                            *; 
* Author: LeRoy Bessler (bessler@execpc.com)                     *;
******************************************************************;

%macro XLSelect(WorkSheet=,Row=,ToRow=,Column=,ToColumn=);

data _null_;
file &DDEfileref;
%if %length(&WorkSheet) ne 0 %then %do;
put %unquote(%str(%'[workbook.activate("&WorkSheet")]%'));
%end;
%let TopRow    = %str();
%let BottomRow = %str();
%let LeftCol   = %str();
%let RightCol  = %str();
%if %length(&Row) ne 0
%then %let TopRow = R&Row;
%if %length(&ToRow) ne 0
%then %let BottomRow = R&ToRow;
%if %length(&Column) ne 0
%then %let LeftCol = C&Column;
%if %length(&ToColumn) ne 0
%then %let RightCol = C&ToColumn;
%if %length(&BottomRow&RightCol) ne 0
%then %let Select = &TopRow&LeftCol.:&BottomRow&RightCol;
%else %let Select = &TopRow&LeftCol;
put %unquote(%str(%'[select("&Select")]%'));
run;

%mend XLSelect;