
* 01_ColorCodingData.sas *;

* NOTE:
  To use this sample code,
  you can choose to highlight specific code blocks and submit them individually.
  This sample program need not be submitted and run en masse.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS code execution step
  to inspect the results. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output; * no output is created *;

* tell SAS where the macros are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;

* Create a spreadsheet to work with *;
* This PROC EXPORT code was generated with the Export Wizard.
  On your system, the appropriate DBMS= value may be different. *;
PROC EXPORT DATA=SASHELP.CLASS 
            OUTFILE="&FolderForXLSfiles.\TempClass.xls" 
            DBMS=EXCEL2000 REPLACE;
RUN;

%XLStart;

%XLDDEcmdsFileName;

%XLOpen(WorkBook=&FolderForXLSfiles.\TempClass.xls);

* Analyze the contents of a column,
  as preparation to color-coding it. *;

* The author would like the height
  of the basketball hoop raised to, say, 15 feet,
  to eliminate height as an advantage in sport.
  Given that such is unlikely to happen,
  this example will flag height greater than average
  as "Bad" (i.e., an advantage not enjoyed by him). *;

* Get the data from the Height column *;

%XLFileNameForOneColumn(WorkSheet=Class,Column=4,StartRow=2);

data AllHeights;
infile Column;
input height;
run;

* Analyze it *;

proc means data=AllHeights noprint mean;
var height;
output out=AverageHeight mean=AvgHgt;
run;

* Create a list of symbolic variables, ColorIt1-ColorIt19,
  to classify the nineteen height values *;

data _null_;
set AllHeights end=LastCell;
if _N_ eq 1 then set AverageHeight;
if height le AvgHgt
then ColorIt = 'Good';
else ColorIt = 'Bad ';
call symput('ColorIt'||trim(left(_N_)),trim(left(ColorIt)));
if LastCell;
call symput('HowMany',_N_);
run;

* List the symbolic variables in the SAS log *;
* But you will see ALL symbols still in effect
  from prior processing in your current SAS session *;
%put _user_;

******************************************************************;
* Note: If using a dark background color,                        *;
*       use a light foreground color for readability.            *;
*       Foreground color is the text/data color.                 *;
******************************************************************;

* Black on intense, or medium to dark, Blue is hard to read.
  Even on Red, it can be somewhat hard to read.
  In this example,
  we will use White text/data only for the Blue background.
  You could use it for both backgrounds. *;

%XLColorCodeCellsInColumn(
                 Column=4,StartRow=2,
                 ColorCodeSymVarPrefix=ColorIt,
                 ColorCodeCount=&HowMany,
                 GoodXLBackgroundColorNumber=32, /* Blue  */
                 GoodXLForegroundColorNumber=2,  /* White */
                 BadXLBackgroundColorNumber=3,   /* Red   */
                 BadXLForegroundColorNumber=1);  /* Black */

* Move the mouse out of the color-coded column *;
%XLFocus;

* Exit without saving *;
%XLExit;
