
* 02_WorkingWithTheExcelColorPalette.sas *;

* NOTE:
  To use this sample code,
  you can choose to highlight specific code blocks and submit them individually.
  This sample program need not be submitted and run en masse.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS code execution step
  to inspect the results. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output;

* tell SAS where the macros are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;

* Create a spreadsheet to work with *;
* This PROC EXPORT code was generated with the Export Wizard.
  On your system, the appropriate DBMS= value may be different. *;
PROC EXPORT DATA=SASHELP.CLASS 
            OUTFILE="&FolderForXLSfiles.\TempClass.xls" 
            DBMS=EXCEL2000 REPLACE;
RUN;

%XLStart;

%XLDDEcmdsFileName;

%XLOpen(WorkBook=&FolderForXLSfiles.\TempClass.xls);

* Select the cell at Row 7 Column 7 *;
%XLSelect(Row=7,Column=7);

* Color the background of the cell (selected by %XLFocus)
  with Excel color 3 *;
%XLBackgroundColor(Color=3);

* Make XL Color Number 3 a mixture of
  100% Red + 100% Green + 100% Blue *;
* (FYI, the result is White.) *; 
data _null_;
file &DDEfileref;
put '[edit.color(3,255,255,255)]';
run;
* Go to look at the worksheet *;

* Let's use a macro instead of explicitly coding the edit.color function. *;

* Make XL Color Number 2 a mixture of
  0% Red + 0% Green + 0% Blue *;
* (FYI, the result is Black.) *; 
%XLColorPaletteCustomize(
               XLColor1to56=2,
               Red0to255=0,
               Green0to255=0,
               Blue0to255=0);

* Apply the "revised" Excel color 2 to the selected cell *;
%XLBackgroundColor(Color=2);
* Go to look at the worksheet *;

* When colors have been experimented with,
  but you want to get back to the "Factory Shipped Palette",
  here is how to do it. *;

%XLColorsResetToDefaults;

* Go to look at the worksheet
  to verify that all cells have their background color
  restored to the most recent background coloring action *;

* Exit without saving *;
%XLExit;

* Start Excel to get an empty workbook to color *;
%XLStart;

%XLDDEcmdsFileName;

* Note that the workbook contains three worksheets,
  with Sheet1 activated *;

* For illustration purposes, let's delete the extra worksheets *;
* (This is NOT necessary for the purposes of the current exercise) *;
data _null_;
file &DDEfileref;
put '[error(false)]'; * suppress the Excel prompts,
                        and just force the deletes *;
put '[workbook.delete("Sheet2")]';
put '[workbook.delete("Sheet3")]';
run;

%XLMakeColorChart(WorkSheet=Sheet1)
run;

%XLFocus;
* Go to look at the chart.
  The color samples are NOT labelled.
  Also, note that, with the default size of the cells,
  some of the dark colors may be hard (or impossible) to distinguish.
  To get a labelled and re-sized Color Chart,
  use the %XLCustomColorChart macro. *;

* Save the default color chart,
  with its unused title/subtitle rows. *;
%XLSaveAs(WorkBook=&FolderForXLSoutput.\ExcelDefaultColorChartWithNoColorNames.xls);

* Exit *;
%XLExit;
