
* 03_MoreWorkingWithExcelColor.sas *;

* NOTE:
  To use this sample code,
  you can choose to highlight specific code blocks and submit them individually.
  This sample program need not be submitted and run en masse.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS code execution step
  to inspect the results. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output;
%let FolderForCustomColorInfo = C:\BesslerSASandExcel\CustomColorInfo;

* tell SAS where the macros are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;

%XLStart;

%XLDDEcmdsFileName;

 /* Open a color chart that was loaded with Excel Default Colors,
    and then had its cells resized and labeled. */

%XLOpen(WorkBook=&FolderForCustomColorInfo.\DefaultExcelColorChart.xls);

%XLFileNameDistinctForEachCell(
           WorkSheet=Excel Colors,
           StartRow=3,EndRow=9,
           StartColumn=1,EndColumn=8);

libname PermLib "&FolderForCustomColorInfo";

%XLWriteCharDataToCells(
        Data=PermLib.FromColorSpecsChart,
        InVarName=ColorDescription,
        Length=40,
		CellFileRefPrefix=Cell,
        FirstCell=1,LastCell=56);

%XLColorCustomizeTextCellByCell(
        Data=PermLib.FromColorSpecsChart,
        ColorVar=TextXLcolor,
        FirstRow=3,LastRow=9,
        FirstColumn=1,LastColumn=8);

%XLFocus;

%XLSaveAs(WorkBook=&FolderForXLSoutput.\ExcelDefaultColorChartWithLeRBcolorNames.xls);

%XLExit;

%XLStart;

%XLDDEcmdsFileName;

* Open a workbook with title and subtitle,
  and with 56 pre-colored cells,
  but with no color names, just Excel color numbers. *;
* This time the colors will be replaced,
  and matching color names will be provided. *;
* Its subtitle is prepped for later insertion of color names in the cells. *;
%XLOpen(WorkBook=&FolderForCustomColorInfo.\CustomizableExcelColorChart.xls);

libname PermLib "&FolderForCustomColorInfo";

data PermLib.FromNewColorSpecsChart;
infile "&FolderForCustomColorInfo.\ColorSpecsLeRBCustomAndBrowserSafe.txt";
input  @1 ColorDescription     $40.
       @1 WhiteOrBlackTextFlag  $2.
	  @10 xlColorNumber          2.
	  @42 RedValue0to255         3.
	  @46 GreenValue0to255       3.
	  @50 BlueValue0to255        3.;
if RedValue0to255   in (0 51 102 153 204 255)
  and
   GreenValue0to255 in (0 51 102 153 204 255)
  and
   BlueValue0to255  in (0 51 102 153 204 255) 
then BrowserSafe = 'Y';
else BrowserSafe = 'N';
if WhiteOrBlackTextFlag eq 'BL'
then TextXLcolor = 1;
else TextXLcolor = 2;
substr(ColorDescription,1,2) = 'XL';
run;

%XLCustomizeAll56ExcelColors(
            Data=PermLib.FromNewColorSpecsChart,
            ColorNumberVar=xlColorNumber,
            RedVar=RedValue0to255,
            GreenVar=GreenValue0to255,
            BlueVar=BlueValue0to255);

%XLFileNameDistinctForEachCell(
           WorkSheet=Excel Colors,
           StartRow=3,EndRow=9,
           StartColumn=1,EndColumn=8);

%XLWriteCharDataToCells(
        Data=PermLib.FromNewColorSpecsChart,
        InVarName=ColorDescription,
        Length=40,
		CellFileRefPrefix=Cell,
        FirstCell=1,LastCell=56);

%XLColorCustomizeTextCellByCell(
        Data=PermLib.FromNewColorSpecsChart,
        ColorVar=TextXLcolor,
        FirstRow=3,LastRow=9,
        FirstColumn=1,LastColumn=8);

%XLFocus;

* Enlarge the color samples.
  130% may be too large or too small,
  depending on your screen resolution. *;
data _null_;
file &DDEfileref;
put '[zoom(130)]';
run;

%XLSaveAs(WorkBook=&FolderForXLSoutput.\CustomizedExcelColorPalette.xls);

%XLExit;
