
* 04_ComputingInsideTheWorksheet.sas *;

* NOTE:
  To use this sample code,
  you can choose to highlight specific code blocks and submit them individually.
  This sample program need not be submitted and run en masse.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS code execution step
  to inspect the results. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output; * no output is created *;

* tell SAS where the macros are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;

* Create a spreadsheet to work with *;
* This PROC EXPORT code was generated with the Export Wizard.
  On your system, the appropriate DBMS= value may be different. *;
PROC EXPORT DATA=SASHELP.CLASS 
            OUTFILE="&FolderForXLSfiles.\TempClass.xls" 
            DBMS=EXCEL2000 REPLACE;
RUN;

%XLStart;

%XLDDEcmdsFileName;

%XLOpen(WorkBook=&FolderForXLSfiles.\TempClass.xls);

* Insert a column *;
%XLInsertColumn(ToLeftOfThisColumn=4);

* Format the new column with the equivalent of SAS format 4.1 *;
* # is needed in some format specifications, but must be in %str() *;
%XLFormatNumericValue(Format=%str(#0.0));

* FILENAME statement for the header cell of Column 4 *;
* All filerefs have "Cell" as default prefix,        *;
* and have suffix from 1 to RowCount X ColumnCount   *;
%XLFileNameDistinctForEachCell(
           WorkSheet=Class,
           StartRow=1,EndRow=1,
           StartColumn=4,EndColumn=4);

* Create a heading for the new empty column *;
data _null_;
file Cell1;
put 'Lbs./Yr.';
run;

* Sometimes the result of a formula is an error *;
* Such as missing divisor or dividend           *;
*         or zero divisor                       *;
* So, let's force some, to see how we can cope. *;
* In Column 3,
  delete some values, and zero some values *;

%XLSelectColumns(Column=3);

%XLReplaceCellContents(Find=16,Replace=);

%XLReplaceCellContents(Find=12,Replace=0);

* Remove highlighting left from Replace
  so that changed contents can be seen easily *;
%XLSelect(Row=1,Column=3);

* Put quotient of the Weight column and this column in Column 4 *;
data _null_;
file &DDEfileref;
put '[select("R2C4")]'; * Select cell at Row 2 Column 4 *;
put '[formula.fill("=RC[+2]/RC[-1]","R2C4:R20C4")]';
run;

* Find & Select the error messages *;
%XLFindErrorMessages;

* Clear those cells *;
data _null_;
file &DDEfileref;
put '[clear]';
run;

* Instead, plug them with "N/A"                                *;
* (Note that they are still selected before the Find & Replace *;
%XLReplaceCellContents(Find=,Replace=N/A);

* Move the focus *;
%XLFocus(Row=1,Column=4);

* Bold Face for the Column Headings *;
%XLSelectRows(Row=1,ToRow=1);
%XLFont(FontStyle=Bold);

* AutoFit the Columns *;
%XLColumnWidth(Column=1,ToColumn=6,Action=AutoFit);

* Move the focus *;
%XLFocus;

* Select Sex and Age Columns *;
%XLSelectColumns(Column=2,ToColumn=3)
run;

* Center that content *;
%XLAlignment(Horizontal=Center);

* Select Lbs./Yr. Column *;
%XLSelectColumns(Column=4,ToColumn=4);

* Right align that content *;
%XLAlignment(Horizontal=Right);

* Move the focus *;
%XLFocus;

* Select Height & Weight Columns *;
%XLSelectColumns(Column=5,ToColumn=6);

%XLFormatNumericValue(Format=%str(#0.0));

* Move the focus *;
%XLFocus;

* Exit without saving *;
%XLExit;
