
* 06_ExcelSubtotalsAndLevels.sas *;

* NOTE:
  To use this sample code,
  you can choose to highlight specific code blocks and submit them individually.
  This sample program need not be submitted and run en masse.
  Once Excel has been started and a spreadsheet has been opened,
  you can toggle over to the Excel window after each SAS code execution step
  to inspect the results. *;

* These folders can, of course, be named anything you prefer,
  but the expected macros, input data, or output data must be stored there. *;
%let FolderForMacros    = C:\BesslerSASandExcel\macros;
%let FolderForXLSfiles  = C:\BesslerSASandExcel\spreadsheets;
%let FolderForXLSoutput = C:\BesslerSASandExcel\output;

* tell SAS where the macros are stored *;
filename SASandXL "&FolderForMacros";
* search for macros in the order listed *;
options sasautos=(SASandXL sasautos);

options mprint; * list the resolved translation of any macros
                  in the SAS log *;
* Create a spreadsheet to work with *;
* This PROC EXPORT code was generated with the Export Wizard.
  On your system, the appropriate DBMS= value may be different. *;
PROC EXPORT DATA=SASHELP.CLASS 
            OUTFILE="&FolderForXLSfiles.\TempClass.xls" 
            DBMS=EXCEL2000 REPLACE;
RUN;

%XLStart(ExcelPgm=C:\Program Files\Microsoft Office\Office10\EXCEL.exe);

%XLDDEcmdsFileName;

%XLOpen(WorkBook=&FolderForXLSfiles.\TempClass.xls);

* normally, you would probably sort data with SAS
  before exporting it to Excel *;
%XLSortCellData(Key1=Sex,Sequence1=Ascending,
                Key2=Age,Sequence2=Ascending,
                Key3=Name,Sequence3=Ascending)
run;

* turn off the all-cell highlighting (selection)
  left over from the sort prep *;
%XLFocus(Row=2,Column=1); /* move it to first data cell */

* to run an Excel macro, it seems as if:
  either it has to be in the same XLS file
         as the worksheet to be processed,
  or it is necessary to open the other XLS file first,
     and then open the XLS file to be processed. *;
* so, let's close this XLS file *;
%XLClose;

* open the workbook that contains the macro(s) *;
%XLOpen(WorkBook=&FolderForXLSfiles.\SubTotalsMacro.xls);

* open the workbook to be processed with the macro(s) *;
%XLOpen(WorkBook=&FolderForXLSfiles.\TempClass.xls);

* run the macro *;
* The DDE subtotal function refused to work for this situation.
  that's why I use the pre-recorded macro instead. *;
%XLRunMacro(WorkBook=SubTotalsMacro.xls,
            XLMacro=sasHelpClass_SubtotalsByAgeWithinSex);

* At this point, there is one copy of Excel open,
  with two workbooks open,
  but the target of the next DDE command will the report workbook,
  not the Excel macro workbook *;

* select the row of column headings *;
%XLSelectRows(Row=1);

* highlight the column headings with bold face *;
%XLFont(FontStyle=Bold);

* AutoFit all the columns *;
%XLColumnWidth(Column=1,ToColumn=5,Action=AutoFit);

* save the result with a new XLS file name *;
%XLSaveAs(WorkBook=&FolderForXLSoutput.\sasHelpClassWithSubtotals.xls);

* NOTE: Because there are two levels of the subtotals,
        the Excel subtotal function creates two copies of the Grand Totals.
        The task now is to get rid of one of those two copies. *;

* Excel subtotals, when multi-level,
  produce the (same) grand totals for each level *;
* let's get rid of the extras *;

* find the number of the last row in the worksheet *;
%XLGetLastRow(WorkSheet=CLASS,
              MacVarForLastRowNumber=LastRow);

%put Last Row Number is &LastRow;

* there are only two sets of the Grand Totals *;
* we will keep the last set *;
data _null_;
LastRow = &LastRow;
SecondLastGrandTotal = LastRow - 1;
call symput('SecondLastGrandTotal',trim(left(SecondLastGrandTotal)));
run;

%put Second Last Grand Total is Row Number &SecondLastGrandTotal;

* delete the extra row *;
%XLDeleteRow(Row=&SecondLastGrandTotal);

* the DDE command to control cell content alignment
  does not support Indent *;
* one solution is to pre-format the column manually,
  with an Indent,
  then load the column with the data,
  and
  then use the DDE command to align left those cells
  that should not be indented. *;
* since pre-formatting a spreadsheet to load it
  may be unacceptable or unwanted. *;
* there is a "direct" way to control indentation *;

* we will indent some cells in the Sex column,
  which also contains labels for "F Total" and "M Total" *;

* for this to work, you need to know
  which specific cells you want to indent,
  versus which you want to unindent. *;
* If the row position of the cells to be indented
  is not a priori predictable,
  you can read the cell content from that column
  into a SAS data set and analyze it.
  Based on content,
  you can dynamically assign values to macro variables
  to flag which cells (row numbers) to indent.
  A similar technique is used later for color coding content. *;

%XLSelect(Row=2,Column=2,ToRow=15,ToColumn=2);

* establish a prefix of three underscores in cells to indent *;
* use MatchEntireCellContents=Y (actually the macro default)
  so that "F Total" does not get the prefix *;
%XLReplaceCellContents(Find=F,Replace=___Female,
                       MatchEntireCellContents=Y);
 
* change the color of the three underscores to white *;
* (if cell background color is not white,
  use the FontColorNumber macro option to match it) *;
%XLHideTheCellDataPrefix(PrefixLength=3);

%XLSelect(Row=17,Column=2,ToRow=32,ToColumn=2);

* same for Males *;
%XLReplaceCellContents(Find=M,Replace=___Male,
                       MatchEntireCellContents=Y);
 
* same for Males *;
%XLHideTheCellDataPrefix(PrefixLength=3);

%XLSelectColumns(Column=2);

* customize the Total labels *;
%XLReplaceCellContents(Find=F Total,Replace=All Females);
%XLReplaceCellContents(Find=M Total,Replace=All Males);

%XLColumnWidth(Column=2,ToColumn=2,Action=AutoFit);

%XLFocus;

* collapse the detail,
  showing only two levels of subtotals,
  and the grand totals *;

%XLShowLevel(LowestLevel=3);

* collapse the lower level subtotals,
  showing only the higher levels of subtotals,
  and the grand totals *;

%XLShowLevel(LowestLevel=2);

* collapse all subtotals,
  showing only the grand totals *;

%XLShowLevel(LowestLevel=1);

* rather than stepping back up the hierarchy,
  just reopen everything, including the detail *;

%XLShowLevel(LowestLevel=4);

* Save the customized WorkBook *;
%XLSave;

* Close that WorkBook *;
%XLClose;

* Close the Excel Macros WorkBook *;
%XLClose;

* Exit *;
%XLExit;
